#
# Calling sytax:
#
#   python image.py <param file> [--restart=<restart value>]
#
# optional keyword "restart" can have the following values:
# - readin
# - clean
# - smooth
# - mask
# - reclean
#
# ie: python image.py <param file> --restart=clean
#
# The program will output various miriad directories from each task
# as well as a log (galaxy.image.log) that saves the miriad output.
#
# Steps run by the program:
#
# 1. readin
#    -> *.map [keep]
#    -> *.beam [keep]
#
# 2. clean
#    -> *.clean
#    -> *.restor
#    -> *.cc.restor
#    -> *.resid.restor
#
# 3. smooth
#    -> *.restor.convol
#    -> *.restor.convol.hanning
#    -> *.blank
#
# 4. mask
#    -> *.region [keep?]
#    -> *.mask [keep]
#
# 5. reclean
#    -> *.mclean [keep]
#    -> *.mrestor [keep]
#    -> *.cc.mrestor [keep]
#    -> #.resid.mrestor [keep]
#
#
#
# To Add:
#   
#   1. Figure out how to write to a log and have the input scroll
#      (quickly) in the terminal as well, instead of in big chunks.
#  

import os
import sys
import math
import immodules


# check if miriad is able to run:
if os.path.expandvars("$MIRBIN") == "$MIRBIN":
   print ""
   print "MIRIAD tasks cannot be called. " + \
         "Source MIRRC or MIRRC.sh in the MIRIAD directory."
   print ""
   sys.exit(0)


# intialize variables for this scope
pfile = ""
restart = ""
noise = 0.0
lfchans = []
galaxy = ""
log = ""



# check calling sequence and set variables
if len(sys.argv) == 2:
    pfile = sys.argv[1]

elif len(sys.argv) == 3:
    pfile = sys.argv[1]

    tmp = sys.argv[2]
    tmp = tmp.split('=')
    value = tmp[1]
    value = value.lower()

    if value == "readin" or value == "clean" or value == "smooth" or \
       value == "mask" or value == "reclean":
        restart = value

    else:
        print value + " is not a valid restart point."
        print "Please choose: readin clean smooth mask reclean"
        sys.exit(0)

else:
    print "Calling sequence: python " + sys.argv[0] + \
          " <param file> [--restart=<step>]"
    sys.exit(0)


# check that parameter file actually exists.
if not os.path.exists(pfile):
    print "Parameter file " + pfile + " does not exist."
    sys.exit(0)


f = open(pfile,mode="r")
# read file and set parameters
for line in f:
   if line[0] == "#":
      # ignore this line
      continue
   else:
      this = line.split()
      
      # search for = in list
      try:
         i = this.index("=")
      except ValueError:
         # this line does not have an =
         continue
      
      if len(this) != 3:
         continue
      
      param = this[0].lower()
      value = this[2].lower()
      
      
      # stupid switch statement
      if param == "galaxy":
         galaxy = value
         
      elif param == "noise":
         if value:
            noise = float(value) # in mJy!
            
            
      elif param == "lfchans":
         tmp = value.split(',')
         if len(tmp) >= 2:
            lfchans = [int(tmp[0]), int(tmp[1])]
         if len(tmp) >= 4:
            lfchans.append(int(tmp[2]))
            lfchans.append(int(tmp[3]))
         if len(tmp) >= 6:
            lfchans.append(int(tmp[4]))
            lfchans.append(int(tmp[5]))
         if len(tmp) >= 8:
            lfchans.append(int(tmp[6]))
            lfchans.append(int(tmp[7]))
#         if len(tmp) == 2:
#             lfchans = [int(tmp[0]), int(tmp[1])]
#         if len(tmp) == 4:
#             lfchans = [int(tmp[0]), int(tmp[1]), int(tmp[2]), int(tmp[3])]
#         if len(tmp) ==
                        
      else:
          print "Paramter " + param + " is not valid in " + pfile + "."
          sys.exit(0)
                  



if not(galaxy):

    print "No galaxy parameter given."
    print "Add this to the parameter file to continue:"
    print "galaxy = <name>"
    sys.exit(0)


# set up dictionary to keep track of files that will be created.
createdFiles = {"readin":  [galaxy + ".map", \
                            galaxy + ".beam"],
                "clean":   [galaxy + ".clean",\
                            galaxy + ".restor", \
                            galaxy + ".resid.restor", \
                            galaxy + ".restor.linmos", \
                            galaxy + ".cc.restor"], \
                "smooth":  [galaxy + ".cc.restor.convol", \
                            galaxy + ".cc.restor.convol.hanning"], \
                "mask":    [galaxy + ".region",\
                            galaxy + ".mask"], \
                "reclean": [galaxy + ".mclean", \
                            galaxy + ".mrestor", \
                            galaxy + ".resid.mrestor",\
                            galaxy + ".mrestor.linmos", \
                            galaxy + ".cc.mrestor"] }



# remove old files.
save = restart
if restart == "readin":
    for file in createdFiles[restart]:
        if os.path.exists(file):
            os.system("rm -r " + file)
    restart = "clean"

if restart == "clean":
    for file in createdFiles[restart]:
        if os.path.exists(file):
            os.system("rm -r " + file)
    restart = "smooth"    

if restart == "smooth":
    for file in createdFiles[restart]:
        if os.path.exists(file):
            os.system("rm -r " + file)
    restart = "mask"    

if restart == "mask":
    for file in createdFiles[restart]:
        if os.path.exists(file):
            os.system("rm -r " + file)
    restart = "reclean"    

if restart == "reclean":
    for file in createdFiles[restart]:
        if os.path.exists(file):
            os.system("rm -r " + file)

restart = save


# set up the log.
if restart:
   immodules.resetLog(galaxy, restart)



# now figure out where to start.
if os.path.exists(galaxy + ".map") and os.path.exists(galaxy + ".beam") and \
   os.path.exists(galaxy + ".mask"):
    # no masking needed - it's already done!

    if not(lfchans):
        lfchans = immodules.getLFChans(galaxy)

    print "Line Free Channels are: " + str(lfchans)

    if not(noise):
        noise = immodules.getNoise(galaxy, lfchans=lfchans)

    print "Noise is: " + str(noise) + " mJy."
    
    if immodules.doStep(createdFiles["reclean"]):
        immodules.clean(galaxy, 2.5*noise, mask = galaxy + ".mask")
        

# otherwise, start from the beginning and see if the files exist.
else:

    if immodules.doStep(createdFiles["readin"]):
        #remove any existing log file
        if os.path.exists(galaxy + ".image.log"):
            os.system("rm -r " + galaxy + ".image.log")
        if os.path.exists(galaxy + ".isubim.1.fits") and \
           os.path.exists(galaxy + ".bsubim.1.fits"):
           immodules.readIn(galaxy)
        else:
           if not(os.path.exists(galaxy + ".isubim.1.fits")):
              print "File "+ galaxy + \
                    ".isubim.1.fits does not exist."
           else:
              print "File "+ galaxy + \
                    ".bsubim.1.fits does not exist."
           sys.exit(0)

    # once the map is read in, set lfchans and noise if necessary.    
    if not(lfchans):
        lfchans = immodules.getLFChans(galaxy)

    print "Line Free Channels are: " + str(lfchans)

    if not(noise):
        noise = immodules.getNoise(galaxy, lfchans=lfchans)

    print "Noise is: " + str(noise) + " mJy."

    
    if immodules.doStep(createdFiles["clean"]):
        immodules.clean(galaxy, 3*noise)
    
    if immodules.doStep(createdFiles["smooth"]):
        immodules.smooth(galaxy)
    
    if immodules.doStep(createdFiles["mask"]):
        immodules.mask(galaxy, noise)
    
    if immodules.doStep(createdFiles["reclean"]):
        immodules.clean(galaxy, 2.5*noise, mask = galaxy + ".mask")
    



print "Done!"
